#!/bin/bash

################################################################################
##                                                                            ##
## $Id:: validate-metadata.sh 2625 2010-02-28 22:59:23Z michalr             $ ##
##                                                                            ##
## Skript pro validaci archivu s metadaty pro digitalni knihovnu. Testovana   ##
## je struktura archivu a validita XML metadat.                               ##
##                                                                            ##
##                                                                            ##
## Navratova hodnota:                                                         ##
##     0       - archiv je validni                                            ##
##     1-127   - chyba ve strukture nebo obsahu archivu                       ##
##     128-255 - chyba pri provadeni validace                                 ##
##                                                                            ##
################################################################################


##
## Pomocne funkce.
##

# Uschovani chybovych hlaseni do souboru.
function printerror() {
    echo "#####" >>"${lasterrorfile}" 2>&1
    echo "##### CHYBA: ${1}" >>"${lasterrorfile}" 2>&1
    echo "#####" >>"${lasterrorfile}" 2>&1
}

# Vypis varovnych hlaseni na standardni chybovy vystup.
function printwarning() {
    echo "#####" 1>&2
    echo "##### VAROVANI: ${1}" 1>&2
    echo "#####" 1>&2
}

# Ukonceni skriptu s navratovou hodnotou zadanou v argumentu. Pred ukoncenim 
# skriptu je odstranen docasny adresar.
function cleanupandexit() {
    if test -n "${tmpdir}" -a -d "${tmpdir}"; then
    	rm -r "${tmpdir}"
    fi
    if test -n "${lasterrorfile}" -a -f "${lasterrorfile}"; then
    	cat "${lasterrorfile}" 1>&2
    	rm "${lasterrorfile}"
    fi
    exit "${1}"
}


##
## Hlavni vykonna cast.
##

# Vola se s prave ctyrmi argumenty.
test $# -eq 6 || \
    { echo 'Pouziti: '$(basename ${0})' <archiv_k_otestovani> <meta.xml_XML_schema> <references.xml_XML_schema> <metadata_na_text_XSL> <text_na_XeLaTeX_konvertor> <adresar_se_XeLaTeXovymi_styly>' 1>&2; exit 128;}

##
## Vytvoreni docasneho souboru pro ulozeni posledni chybove zpravy pro 
## zobrazeni koncovemu uzivateli.
##
lasterrorfile=$(mktemp -t validate-metadata.lasterror.XXXXXXXXXXXXXX) || \
    { echo 'Nepodarilo se vytvorit docasny soubor na chybova hlaseni.' 1>&2; exit 129;}

# Nechybova hlaseni o prubehu skriptu zahazovat.
exec >/dev/null

# Testovany archiv se zadava jako prvni argument skriptu.
filename="${1}"
test -n "${filename}" -a -f "${filename}" -a -r "${filename}" || \
    { printerror 'Nepodarilo se otevrit archiv k otestovani'; cleanupandexit 130;}
filename="$(readlink -f ${1})" # Ulozeni absolutni cesty k archivu.

# XML schema clankovych metadat se zadava jako druhy argument skriptu.
metaschema="${2}"
test -n "${metaschema}" -a -f "${metaschema}" -a -r "${metaschema}" || \
    { printerror 'Nepodarilo se otevrit XML schema pro validaci souboru meta.xml.'; cleanupandexit 131;}
metaschema="$(readlink -f ${2})" # Ulozeni absolutni cesty ke schematu.

# XML schema seznamu referenci se zadava jako treti argument skriptu.
referencesschema="${3}"
test -n "${referencesschema}" -a -f "${referencesschema}" -a -r "${referencesschema}" || \
    { printerror 'Nepodarilo se otevrit XML schema pro validaci souboru references.xml.'; cleanupandexit 132;}
referencesschema="$(readlink -f ${3})" # Ulozeni absolutni cesty ke schematu.

# XSL pro konverzi XML metadat na text se zadava jako ctvrty argument skriptu.
totextstylesheet="${4}"
test -n "${totextstylesheet}" -a -f "${totextstylesheet}" -a -r "${totextstylesheet}" || \
    { printerror 'Nepodarilo se otevrit XSL pro konverzi XML metadat na text.'; cleanupandexit 133;}
totextstylesheet="$(readlink -f ${4})" # Ulozeni absolutni cesty ke schematu.

# Cesta k prekladaci textove podoby metadat na XeLaTeXovy zdrojak se zadava jako paty argument skriptu.
texttoxelatexproc="${5}"
test -n "${texttoxelatexproc}" -a -f "${texttoxelatexproc}" -a -r "${texttoxelatexproc}" -a -x "${texttoxelatexproc}" || \
    { printerror 'Nepodarilo se otevrit prekladac textove podoby metadat na XeLaTeX.'; cleanupandexit 134;}
texttoxelatexproc="$(readlink -f ${5})" # Ulozeni absolutni cesty ke skriptu.
texttoxelatexmapfile="$(dirname $(readlink -f ${texttoxelatexproc}))/tex.txt" # Ulozeni absolutni cesty k map souboru.

# Adresar se styly pro XeLaTeX se zadava jako sesty argument skriptu.
texdatadir="${6}"
test -n "${texdatadir}" -a -d "${texdatadir}" || \
    { printerror 'Nepodarilo se otevrit adresar se styly pro XeLaTeX.'; cleanupandexit 135;}
texdatadir="$(readlink -f ${6})" # Ulozeni absolutni cesty k adresari se styly pro XeLaTeX.

# Vytvoreni docasneho adresare, ve kterem bude probihat validace.
tmpdir=$(mktemp -d -t validate-metadata.XXXXXXXXXXXXXX) || \
    { printerror 'Nepodarilo se vytvorit docasny adresar pro ulozeni validovanych dat.'; cleanupandexit 136;}

# Archiv k otestovani je zkopirovan do docasneho adresare.
cp ${filename} ${tmpdir}/ || \
    { printerror 'Nepodarilo se zkopirovat archiv do docasneho adresare.'; cleanupandexit 137;}

# Prechod do docaseho adresare.
pushd ${tmpdir} || \
    { printerror 'Nepodarilo se prejit do docasneho adresare.'; cleanupandexit 138;}

    # Rozbaleni archivu.
    unzip ${filename} &>"${lasterrorfile}" || \
        { printerror 'Nepodarilo se rozbalit ZIP archiv.'; cleanupandexit 1;}
    >"${lasterrorfile}" # Smazani logu, neobsahuje chyby.

    # Kontrola, zda archiv obsahuje prave jeden adresar.
    test $(ls | grep -iv "^$(basename ${filename})$" | wc -l) -eq 1 || \
        { printerror 'Archiv ma obsahovat prave jeden korenovy adresar.'; cleanupandexit 2;}

    # Kontrola nazvu adresare s metadaty po rozbaleni.
    # Ma byt: <rocnik>-<rok>-<cislo>
    echo $(ls | grep -iv "^$(basename ${filename})$") | egrep '^[[:digit:]]{3}-[[:digit:]]{4}-[[:digit:]]+$' || \
        { printerror 'Korenovy adresar nema korektni nazev.'; cleanupandexit 3;}

    # Prechod do korenoveho adresare rozbaleneho archivu.
    pushd $(ls | grep -iv "^$(basename ${filename})$") || \
        { printerror 'Nepodarilo se prejit do rozbaleneho korenoveho adresare.'; cleanupandexit 139;}

        # Kontrola pritomnosti adresare se zdrojovymi texty.
        test -d source || \
            { printwarning 'Chybi korenovy adresar "source".';}

        # Overeni, ze je pritomen alespon jeden adresar s clankovymi daty.
        test $(ls -d \#* | wc -l) -ge 1 || \
            { printerror 'Chybi adresare s clanky ("#1/", "#2/", ...).'; cleanupandexit 4;}

        # Kontrola jednotlivych clanku.
        for i in $(seq 1 $(ls -d \#* | wc -l)); do 

            # Kontrola, zda jsou vsechny clanky oznaceny po jedne 
            # inkrementovanym poradovym cislem a zadne cislo nechybi.
            test -d \#${i}/ || \
                { printerror 'Chybi adresar "#'${i}'/".'; cleanupandexit 5;}

            # Prechod do podadresare konkretniho clanku.
            pushd \#${i}/ || \
                { printerror 'Nepodarilo se prejit do podadresare clanku.'; cleanupandexit 140;}

                # Kontrola pritomnosti adresare se zdrojovymi texty.
                if test -d source; then

                    # Kontrola pritomnosti korektne pojmenovaneho hlavniho zdrojoveho textu clanku.
                    if test $(ls source/mainsource-*.tex | wc -l) -eq 1; then

                        # Kontrola pritomnosti prave jednoho hlavniho zdrojoveho textu clanku.
                        test -f source/mainsource-*.tex || \
                            printwarning 'Chybi korenovy TeXovy soubor "mainsource-<identifikator>.tex" v adresari "#'${i}'/source/".'

                    else

                        printwarning 'V adresari "#'${i}'/source/" ma byt prave jeden korenovy TeXovy soubor "mainsource-<identifikator>.tex".'

                    fi

                else

                    printwarning 'Chybi adresar "#'${i}'/source/".'

                fi

                # Kontrola pritomnosti PDF s fulltextem clanku.
                test -f source.pdf || \
                    { printerror 'Chybi PDF fulltextu clanku "source.pdf" v adresari "#'${i}'/".'; cleanupandexit 6;}

                # Kontrola pritomnosti clankovych XML metadat.
                test -f meta.xml || \
                    { printerror 'Chybi clankova metadata "meta.xml" v adresari "#'${i}'/".'; cleanupandexit 7;}

                # Validace clankovych XML metadat.
                xmllint meta.xml -schema ${metaschema} &>"${lasterrorfile}" || \
                    { printerror 'Clankova metadata "meta.xml" v adresari "#'${i}'/" nejsou validni.'; cleanupandexit 8;}
                >"${lasterrorfile}" # Smazani logu, neobsahuje chyby.

                # Validace obsahu clankovych XML metadat prekladem XeLaTeXem.
                cp ${texdatadir}/* . || \
                    { printerror 'Nepodarilo se zkopirovat XeLaTeXove styly.'; cleanupandexit 141;}
                xsltproc -novalid ${totextstylesheet} meta.xml | "${texttoxelatexproc}" "${texttoxelatexmapfile}" >meta-xelatex-input.tex || \
                    { printerror 'Nepodarilo se vytvorit XeLaTeXovy zdrojak z meta.xml.'; cleanupandexit 142;}
                xelatex -interaction=nonstopmode meta-xelatex-input.tex &>"${lasterrorfile}" || \
                    { cat "${lasterrorfile}" 1>&2; printwarning 'Textovy obsah clankovych metadata "meta.xml" v adresari "#'${i}'/" neni prelozitelny XeLaTeXem.';}
                >"${lasterrorfile}" # Smazani logu, neobsahuje chyby.

                # Kontrola pritomnosti XML seznamu referenci (soubor neni povinny).
                if test -f references.xml; then

                    # Validace XML seznamu referenci.
                    xmllint references.xml -schema ${referencesschema} &>"${lasterrorfile}" || \
                        { printerror 'Seznam referenci "references.xml" v adresari "#'${i}'/" neni validni.'; cleanupandexit 9;}
                    >"${lasterrorfile}" # Smazani logu, neobsahuje chyby.

                    # Validace obsahu XML seznamu referenci prekladem XeLaTeXem.
                    cp ${texdatadir}/* . || \
                        { printerror 'Nepodarilo se zkopirovat XeLaTeXove styly.'; cleanupandexit 143;}
                    xsltproc -novalid ${totextstylesheet} references.xml | "${texttoxelatexproc}" "${texttoxelatexmapfile}" >references-xelatex-input.tex || \
                        { printerror 'Nepodarilo se vytvorit XeLaTeXovy zdrojak z references.xml.'; cleanupandexit 144;}
                    xelatex -interaction=nonstopmode references-xelatex-input.tex &>"${lasterrorfile}" || \
                        { cat "${lasterrorfile}" 1>&2; printwarning 'Textovy obsah clankovych metadata "references.xml" v adresari "#'${i}'/" neni prelozitelny XeLaTeXem.';}
                    >"${lasterrorfile}" # Smazani logu, neobsahuje chyby.

                fi

            popd # Navrat z podadresare konkretniho clanku.

        done # Konec kontroly jednotlivych clanku.

    popd # Navrat z korenoveho adresare rozbaleneho archivu.

popd # Navrat z docaseho adresare.

# Archiv je validni.
>"${lasterrorfile}" # Smazani logu, neobsahuje chyby.
cleanupandexit 0 # Uspesne ukonceni programu.

# vim: set sw=4 tabstop=4 expandtab :
